Object.defineProperty(exports, "__esModule", {
  value: true
});
exports['default'] = build;

var _ora = require('ora');

var _ora2 = _interopRequireDefault(_ora);

var _logSymbols = require('log-symbols');

var _logSymbols2 = _interopRequireDefault(_logSymbols);

var _webpack2 = require('./webpack');

var _webpack3 = _interopRequireDefault(_webpack2);

var _loadCorenConfig = require('./load-coren-config');

var _loadCorenConfig2 = _interopRequireDefault(_loadCorenConfig);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var error = _utils.color.error;


var runWebpack = function runWebpack(compiler, msg) {
  return function () {
    var spinner = (0, _ora2['default'])(msg).start();
    return new Promise(function (resolve, reject) {
      compiler.run(function (err, stats) {
        if (err) {
          return reject(err);
        }

        var jsonStats = stats.toJson();

        if (jsonStats.errors.length > 0) {
          var _error = new Error(jsonStats.errors[0]);
          _error.errors = jsonStats.errors;
          _error.warnings = jsonStats.warnings;
          return reject(_error);
        }
        spinner.stopAndPersist({
          symbol: _logSymbols2['default'].success,
          text: msg
        });
        resolve(jsonStats);
      });
    });
  };
};

function build(_ref) {
  var dir = _ref.dir,
      env = _ref.env,
      clientWebpackPath = _ref.clientWebpackPath;

  var dev = env !== 'production';
  var config = (0, _loadCorenConfig2['default'])(dir);

  var _webpack = (0, _webpack3['default'])({ dir: dir, corenConfig: config, dev: dev, clientWebpackPath: clientWebpackPath }),
      clientCompiler = _webpack.clientCompiler,
      serverCompiler = _webpack.serverCompiler;

  var runServerWebpack = runWebpack(serverCompiler, 'Building server side webpack');

  return runServerWebpack().then(function () {
    if (!dev) {
      return runWebpack(clientCompiler, 'Building client side webpack')();
    }
  })['catch'](function (err) {
    throw new Error(error(err));
  });
}
module.exports = exports['default'];