var _cheerio = require('cheerio');

var _cheerio2 = _interopRequireDefault(_cheerio);

var _fs = require('fs');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _corenWorkingSpace = require('./coren-working-space');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var getEntryHtml = function getEntryHtml(reqPath, rootPath) {
  var filePath = _path2['default'].join((0, _corenWorkingSpace.getSsrDir)(rootPath), reqPath);
  if ((0, _fs.existsSync)(filePath) && (0, _fs.lstatSync)(filePath).isDirectory()) {
    filePath = String(filePath) + '/index.html';
  }

  return (0, _fs.readFileSync)(filePath, 'utf8');
};

var findCorenScript = function findCorenScript($) {
  var scriptTag = void 0;
  $('script').get().forEach(function (script) {
    if ((0, _lodash.has)(script.attribs, 'data-coren')) {
      scriptTag = script;
    }
  });
  return scriptTag;
};

var updatePreloadedState = function updatePreloadedState($, newState) {
  if ((0, _lodash.isEmpty)(newState)) {
    return $;
  }

  var script = findCorenScript($);
  if (script) {
    var preloadStateTxt = script.children[0].data;
    var preloadState = JSON.parse(preloadStateTxt.replace('window.__PRELOADED_STATE__ =', ''));
    var mergeState = Object.assign({}, preloadState, newState);
    script.children[0].data = 'window.__PRELOADED_STATE__ = ' + JSON.stringify(mergeState);
  } else {
    // insert one
    $('head').append('<script data-coren="">window.__PRELOADED_STATE__ = ' + String(JSON.stringify(newState)) + '</script>');
  }
  return $;
};

module.exports = function (rootPath) {
  return function () {
    function corenMiddleware(req, res, next) {
      var setHead = void 0;
      var preloadedState = void 0;

      res.sendCoren = function (reqPath) {
        var $ = _cheerio2['default'].load(getEntryHtml(reqPath, rootPath));
        if (preloadedState) {
          $ = updatePreloadedState($, preloadedState);
        }
        if (setHead) {
          var api = {
            append: function () {
              function append(content) {
                $('head').append(content);
              }

              return append;
            }()
          };
          setHead(api);
        }
        res.send($.html());
      };

      res.setHead = function (cb) {
        setHead = cb;
        return res;
      };

      res.setPreloadedState = function (obj) {
        preloadedState = obj;
        return res;
      };
      next();
    }

    return corenMiddleware;
  }();
};